.module AI_System

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;      INCLUDES        ;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "entity.inc"
.include "sprites.inc"  
.include "patrol.inc"
.include "IA.inc"
.include "Animaciones.inc"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones Propias ;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.globl man_game_entity_destroy
.globl man_game_create_template_entity
.globl init_player_shot
.globl generar_bala
.globl HUD_SIZE_H
.globl man_entity_setCooldown
.globl wave_finished
.globl enemies_onscreen
.globl man_game_entity_destroy
.globl generar_objeto
.globl sys_collision_enemy_point
.globl decrease_enemies
.globl blink_render_enemy
.globl show_victory_screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones ;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_AI_Update_One_Entity'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Actualiza el comportamiento de la entidad
;;
sys_ai_update_one_entity::
    ld  a, Ent_TYPE(ix)
    and #e_type_dead
    cp  #e_type_dead                ; Si esta marcada como muerta no se actualiza
    jp  z, fin_ai

        ld hl, #fin_ai
        push hl

        ld h, Ent_AI2(ix)           ; Obtenemos su comportamiento
        ld l, Ent_AI1(ix)           ; Obtenemos su comportamiento
        
        jp (hl)                     ; Llamamos al comportamiento

    fin_ai::
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_AI_Update'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Llama a la funcion de entidad que 
;; recorre todo el array de entidades
;; para un componente en concreto (D)
;; y luego llama a la funcion en (HL)
;;
sys_ai_update::
    ld hl, #sys_ai_update_one_entity
    ld (forall_function), hl
    ld a, #e_cmp_ai
    ld (forall_filter), a                             ; Guardamos el componente en forall_filter
    call man_entity_forall_matching
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; COMPORTAMIENTOS IA
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'autodestroy_ai'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direcciona de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Se encarga de destruir la entidad
;; cuando su tiempo de vida en la pantalla
;; ya ha pasado (Bala)
;;
autodestroy_ai::
    ld  a, Ent_PATROL1(ix)
    cp  #0
    jp  nz, no_tengo_comprobar
        call sys_ia_check_bullet_out_of_bounds
    no_tengo_comprobar:
    ld  a, Ent_AI_COUNT(ix)           ; Recuperamos el contador de tiempo
    dec a                   ; Y lo decrementamos
    ld  Ent_AI_COUNT(ix), a           ; Guardamos el nuevo valor
    cp  #0
    jp  nz, saltar_autodestroy  ; Cuando sea cero, marcamos la entidad para destruir
        ld   a, Ent_PATROL1(ix)
        cp   #0
        jp   nz, hay_que_borrar_bala
            call sys_ia_shot_desaparece
            jp   saltar_autodestroy
        hay_que_borrar_bala:
        call man_game_entity_destroy
    saltar_autodestroy:
ret

sys_ia_shot_desaparece::

    ld   hl, #sprites_shotdead
    call man_entity_setSprite

    ld   a, Ent_COMPONENTS(ix)
    sub  #e_cmp_collider
    sub  #e_cmp_movable
    ld   Ent_COMPONENTS(ix), a

    ld  Ent_AI_COUNT(ix), #5
    ld  Ent_PATROL1(ix),  #1

    ret

sys_ia_check_bullet_out_of_bounds::
    ; Comprobamos si está fuera del límite en X
    ld  a, Ent_X(ix)
    add a, Ent_VX(ix)
    cp  #8
    jp  nc, no_es
    call sys_ia_shot_desaparece
    jp   fin_check
    no_es:
    cp #147
    jp c, no_es2
    call sys_ia_shot_desaparece
    jp   fin_check
    no_es2:
    
    ; Comprobamos si está fuera del límite en Y
    ld a,  Ent_Y(ix)
    add a, Ent_VY(ix)
    cp #32
    jp  nc, no_es3
    call sys_ia_shot_desaparece
    jp   fin_check
    no_es3:
    cp #200 - 20
    jp  c, no_es4
    call sys_ia_shot_desaparece
    no_es4:

    fin_check:
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'iniciar_moveto'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direccion de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Recoge la posicion actual del jugador
;; y los guarda en los objetivos de la entidad
;;
sys_ia_seguir_jugador::
    ld   iy, (direccion_jugador)                  ;; Apuntamos a la Entidad del JUGADOR
    ld   a , Ent_X(iy)
    ld   Ent_OBJX(ix), a                    ;; Guardamos la posicion de X del jugador en la posicion Objetivo de X
    ld   a , Ent_Y(iy)
    ld   Ent_OBJY(ix), a                    ;; Guardamos la posicion de Y del jugador en la posicion Objetivo de Y

    ld   a, Ent_PATROL1(ix)
    cp   #1
    jp   nz, no_boss
        ld   Ent_AI1(ix), #<sys_ia_move_to
        ld   Ent_AI2(ix), #>sys_ia_move_to      
        jp   fin_sys_ia_seguir_jugador
    no_boss:
    ld   Ent_AI1(ix), #<sys_ia_move_to_saltar_disparo
    ld   Ent_AI2(ix), #>sys_ia_move_to_saltar_disparo      ;; Cambiamos el comportamiento para que inicie el move_to
    fin_sys_ia_seguir_jugador:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'move_to'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direccion de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Se mueve en direccion hasta las
;; coordenadas objetivos
;;
sys_ia_move_to::
    push ix
    call sys_ia_shot
    pop  ix

    sys_ia_move_to_saltar_disparo:
    ;;  EJE X
    ld  a, Ent_X(ix)        ;; Posicion actual de la entidad
    ld  h, Ent_OBJX(ix)     ;; Posicion objetivo

    cp  h                   ;; Restamos el valor que buscamos // Puede dar 0 / Menor / Mayor
    jp  z,  ceroX            
    jp  nc, menorX
    ;; Mayor
        cp #152-SPR_SOLDADO_W*2
        jp c,no_limit_right
            ld  Ent_VX(ix), #0
        jp fin_X     
        no_limit_right:
            ld  Ent_VX(ix), #1
        jp  fin_X
    menorX:
    ;; Menor
        cp #9
        jp nc,no_limit_left
            ld  Ent_VX(ix), #0
        jp fin_X     
        no_limit_left:
            ld  Ent_VX(ix), #-1
        jp  fin_X
    ceroX :
        ld  Ent_VX(ix), #0      ;; Cero
    fin_X:

    ;;  EJE Y
    ld  a, Ent_Y(ix)        ;; Posicion actual de la entidad
    ld  h, Ent_OBJY(ix)     ;; Posicion objetivo

    cp  h                   ;; Restamos el valor que buscamos // Puede dar 0 / Menor / Mayor
    jp  z,  ceroY            
    jp  nc, menorY
    ;; Mayor
        cp #199-HUD_SIZE_H-SPR_SOLDADO_H
        jp c,no_limit_down
            ld  Ent_VY(ix), #0
        jp fin_Y      
        no_limit_down:
            ld  Ent_VY(ix), #1
        jp  fin_Y
    menorY:
    ;; Menor
        cp #28
        jp nc,no_limit_up
            ld  Ent_VY(ix), #0
        jp fin_Y      
        no_limit_up:
            ld  Ent_VY(ix), #-1
        jp  fin_Y
    ceroY :
        ld  Ent_VY(ix), #0      ;; Cero
    fin_Y:

    call animaciones        ;; Posible mejora del rendimiento

    ;; Comprobamos si hemos llegado a la posicion objetivo
    ld  a, Ent_VX(ix)
    cp  #0
    jp  nz, no_cambiar_estado
    ld  a, Ent_VY(ix)
    cp  #0
    jp  nz, no_cambiar_estado                   ; Una vez llegamos a la posicion objetivo cambiamos el estado a 
        ld   Ent_ANIM_COUNT(ix), #IA_ANIM_STOP        ; Y la desactivamos para optimizar
        ld a,Ent_AIPREV1(ix)
            cp #<sys_ia_sniper_en_posicion
            jp nz,no_primera_sniper
                ld a,Ent_AIPREV2(ix)
                cp #>sys_ia_sniper_en_posicion
                jp nz,no_primera_sniper
                    ld a,#1
                    ld (ha_llegado),a
            no_primera_sniper:
        ld   Ent_AI1(ix), #<sys_ia_standby    
        ld   Ent_AI2(ix), #>sys_ia_standby
    no_cambiar_estado:

ret

ha_llegado: .db 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_ia_patrol'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direccion de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Se fija una velocidades iniciales de
;; patrulla para vigilar uno de los ejes
;;
sys_ia_patrol::

    ld  l, Ent_PATROL1(ix)      ; Recuperamos el step actual de la patrulla
    ld  h, Ent_PATROL2(ix)

    ld  a, (hl)                 ; Cargamos el primer punto de patrull
    cp  #patrol_invalid_number
    jp  z, resetear_patrulla    ; Si hemos llegado al final, reseteamos la patrulla

    ld  Ent_OBJX(ix), a         ; Lo guardamos en el objetivo de X
    inc hl
    ld  a, (hl)                 ; Cargamos el segundo punto de patrulla
    ld  Ent_OBJY(ix), a         ; Lo guardamos en el objetivo de Y
    inc hl

    ld  Ent_PATROL1(ix), l
    ld  Ent_PATROL2(ix), h

    ld  Ent_AI1(ix), #<sys_ia_move_to   ; Una vez hemos cargado los puntos de patrulla
    ld  Ent_AI2(ix), #>sys_ia_move_to   ; Cambiamos su comportamiento al move_to

    fin_patrol:
    ret

    resetear_patrulla:      ; Apuntamos el puntero de la patrulla al inicio
        inc hl
        ld   a, (hl)
        ld  Ent_PATROL1(ix), a  
        inc hl
        ld   a, (hl)
        ld  Ent_PATROL2(ix), a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_ia_shot'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direcciona de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Si la entidad puede disparar en el 
;; momento, lo hará
;;
sys_ia_shot::
    ld   a, Ent_COOLDOWN(ix)                ; Si el contador de espera del enemigo no se ha cumplido
    dec  a                                  ; no podrá efectuar el disparo
    ld   Ent_COOLDOWN(ix), a
    jp   z, disparar
    ret

    disparar:
        ld   a, Ent_COOLDOWN_O(ix)          ; Una vez podemos disparar, volvemos a cargar su contador de espera
        ld   Ent_COOLDOWN(ix), a            ; al valor del contador original que tenia
        call generar_bala                   ; Llamamos al méteodo que se encarga de crear las balas y le pasamos
        ret                                 ; en (IX) la entidad que dispara


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_ia_sniper'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direcciona de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Comportamiento que simula un soldado
;; francotirador que dispara desde lejos
;;

remember_cooldown: .db 0

sys_ia_sniper::
    ld a,Ent_AI_COUNT_O(ix)
    ld (remember_cooldown),a
    ld a,#1
    call man_entity_setCooldown
    ld Ent_AI1(ix), #<sys_ia_move_to_saltar_disparo       ; Movemos a la posicion de francotirador
    ld Ent_AI2(ix), #>sys_ia_move_to_saltar_disparo   
    ret

    sys_ia_sniper_en_posicion:

    ld  a, Ent_PATROL1(ix)
    or  a 
    jp  z, sniper_escondido
        ld   a, Ent_PATROL2(ix)
        cp   #1
        jp   nz,  animacion_hecha2 
            ld   Ent_ANIM1(ix), #<anim_bandido_salir
            ld   Ent_ANIM2(ix), #>anim_bandido_salir
            ld   Ent_ANIM_COUNT(ix), #IA_ANIM_RUN
            ld   Ent_AI1(ix), #<sys_ia_animacion
            ld   Ent_AI2(ix), #>sys_ia_animacion
            ret
        animacion_hecha2: 
        ;TIEMPO DE ESPERA ESCONDIDO
        ;ld a,Ent_AI_COUNT_O(ix)
        ;sla a
        ;ld Ent_AI_COUNT_O(ix),a

        ld   a, Ent_COMPONENTS(ix)
        or   #e_cmp_collider
        ld   Ent_COMPONENTS(ix), a
        ;; VUELVE DEL STANDBY
        ld  a, Ent_AI_COUNT(ix) ; Recuperamos el contador de espera
        dec a                   ; Y lo decrementamos
        ld  Ent_AI_COUNT(ix), a ; Guardamos su nuevo valor
        or  a
        jp  nz, seguimos_disparando
            ld   Ent_PATROL1(ix), #0
            ld   a, Ent_AI_COUNT_O(ix)
            ld   Ent_AI_COUNT(ix), a
            ret
        seguimos_disparando:
        call apuntar_jugador                        ; Apuntamos al jugador antes de disparar
        call sys_ia_shot                            ; Llamamos a disparar en todas las iteraciones de francotirador
        ret     
    sniper_escondido:
        ld   a, Ent_PATROL2(ix)
        cp   #0
        jp   nz,  animacion_hecha 
            ld   Ent_ANIM1(ix), #<anim_bandido_esconderse
            ld   Ent_ANIM2(ix), #>anim_bandido_esconderse
            ld   Ent_ANIM_COUNT(ix), #IA_ANIM_RUN
            ld   Ent_AI1(ix), #<sys_ia_animacion
            ld   Ent_AI2(ix), #>sys_ia_animacion
            ret
        animacion_hecha:
        ld   Ent_AI1(ix), #<sys_ia_standby
        ld   Ent_AI2(ix), #>sys_ia_standby
        ld   Ent_PATROL1(ix), #1
        ld   a, Ent_COMPONENTS(ix)
        sub  #e_cmp_collider
        ld   Ent_COMPONENTS(ix), a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_ia_animacion'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direcciona de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Espera a que la animacion en curso 
;; se complete
;;
sys_ia_animacion::

        ld h, Ent_ANIM2(ix)
        ld l, Ent_ANIM1(ix)
        inc hl
        inc hl
        inc hl
        ld a,(hl)
        cp #0
    jp z, sys_ia_fin_animacion
        ret

    sys_ia_fin_animacion:
    ld   Ent_ANIM_COUNT(ix), #IA_ANIM_STOP
    ld   a, Ent_AIPREV1(ix)
    ld   Ent_AI1(ix), a
    ld   a, Ent_AIPREV2(ix) 
    ld   Ent_AI2(ix), a

    ld   a, Ent_PATROL2(ix)
    cp   #0
    jp   nz, no_es_cero
        ld  Ent_PATROL2(ix), #1
        jp  fin
    no_es_cero:
        ld  Ent_PATROL2(ix), #0
    fin:
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'apuntar_jugador'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direcciona de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Apuntamos el sprite del enemigo al
;; sitio del jugador
;;
apuntar_jugador::

    ld   iy, (direccion_jugador)              ; IY = JUGADOR

    ;; EJES
    ld  h, Ent_Y(ix)                  ; A = EnemigoY
    ld  a, Ent_Y(iy)                  ; H = JugadorY
    add a, #SPR_JUGADOR_H
    cp  h 
    jp  c, fin_lineal
    ld  a, Ent_Y(iy)                  ; H = JugadorY
    sub a, #SPR_JUGADOR_H
    cp  h
    jp  nc, fin_lineal
    ; Estamos en el mismo eje X

    ld  a, Ent_X(ix)                  ; A = EnemigoX
    ld  h, Ent_X(iy)                  ; H = JugadorX
    cp  h
    jp  nc, izquierda
        ld Ent_DIRECCION(ix), #2        ; DERECHA
        jp fin_apuntar
    izquierda:
        ld Ent_DIRECCION(ix), #6        ; IZQUIERDA
        jp fin_apuntar

    fin_lineal:

    ld  h, Ent_X(ix)                  ; A = EnemigoX
    ld  a, Ent_X(iy)                  ; H = JugadorX
    add a, #SPR_JUGADOR_W*2
    cp  h 
    jp  c, diagonales
    ld  a, Ent_X(iy)                  ; H = JugadorX
    sub a, #SPR_JUGADOR_W*2
    cp  h
    jp  nc, diagonales
    ; Estamos en el mismo eje X

    ld  a, Ent_Y(ix)                  ; A = EnemigoY
    ld  h, Ent_Y(iy)                  ; H = JugadorY
    cp  h
    jp  nc, arriba
        ld Ent_DIRECCION(ix), #4        ; ABAJO
        jp fin_apuntar
    arriba:
        ld Ent_DIRECCION(ix), #0        ; ARRIBA
        jp fin_apuntar

    ; Diagonales
    diagonales:
    ld  a, Ent_Y(ix)                  ; A = EnemigoY
    ld  h, Ent_Y(iy)                  ; H = JugadorY
    cp  h
    jp  nc, PY_arriba
        ld Ent_DIRECCION(ix), #4        ; Jugador por debajo
        jp fin_apuntarY
    PY_arriba:                          ; Jugador por encima
    ld Ent_DIRECCION(ix), #0
    fin_apuntarY:

    ld  a, Ent_X(ix)                  ; A = EnemigoX
    ld  h, Ent_X(iy)                  ; H = JugadorX
    cp  h
    jp  c, PY_derecha
        ld a, Ent_DIRECCION(ix)          ; Jugador izquierda
        or a
        jp nz, abajo_iz
        ld Ent_DIRECCION(ix), #7         ; Jugador Arriba izquierda
        jp fin_apuntar
        abajo_iz:           
        ld Ent_DIRECCION(ix), #5         ; Jugador Abajo izquierda
        jp fin_apuntar
    PY_derecha:                          ; Jugador derecha
        ld a, Ent_DIRECCION(ix)             
        or a
        jp nz, abajo_de
            ld Ent_DIRECCION(ix), #1     ; Jugador Arriba derecha
            jp fin_apuntar
        abajo_de:
            ld Ent_DIRECCION(ix), #3     ; Jugador Abajo derecha
    fin_apuntar:

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_ia_standby'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direcciona de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Para la entidad unos segundos antes de
;; devolvele el comportamiento
;;
sys_ia_standby::
    ld  a, Ent_AI_COUNT(ix) ; Recuperamos el contador de espera
    dec a                   ; Y lo decrementamos
    ld  Ent_AI_COUNT(ix), a ; Guardamos su nuevo valor
    cp  #0
    jp  nz, seguimos_standby    ; Si es cero, recuperamos el comportamiento original de le entidad

        ld l, Ent_AIPREV1(ix)
        ld h, Ent_AIPREV2(ix)
        ld a, #<sys_ia_sniper_en_posicion
        cp l
        jp nz, no_es_sniper
        ld a, #>sys_ia_sniper_en_posicion
        cp h
        jp nz, no_es_sniper
            ;; ES SNIPE
            ld a,(ha_llegado)
            cp #1
            jp nz, no_run
                ld a,(remember_cooldown)
                call man_entity_setCooldown
                ld a,#0
                ld (ha_llegado),a
            jp no_run
        no_es_sniper:

        ld   Ent_ANIM_COUNT(ix), #IA_ANIM_RUN
        no_run:
        ld   Ent_AI1(ix), l
        ld   Ent_AI2(ix), h
        ld   a, Ent_AI_COUNT_O(ix)
        ld   Ent_AI_COUNT(ix), a  ; Volvemos a poner el contador de espera del entidad
    seguimos_standby:
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_ia_bouncer'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direcciona de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; simula un comportamiento de rebote
;; con los limtes del mapa
;;
sys_ia_bouncer::
    ; EJE X
    ld  a, Ent_X(ix)
    add Ent_VX(ix)
    cp  #8 + VELOCIDAD_IA_BOUNCER_STROLL
    jp  nc, comprobar_limite_derecho
    ;Límte izquierdo
    ld  Ent_VX(ix), #VELOCIDAD_IA_BOUNCER_STROLL
    ld Ent_ANIM1(ix), #<anim_calabaza_right
    ld Ent_ANIM2(ix), #>anim_calabaza_right
    jp  fin_eje_X

    comprobar_limite_derecho:
    ld  a, Ent_X(ix)
    add Ent_VX(ix)
    cp  #154-SPR_CALABAZA_W*2 - VELOCIDAD_IA_BOUNCER_STROLL
    jp  c, fin_eje_X
    ;Límite derecho
    ld Ent_ANIM1(ix), #<anim_calabaza_left
    ld Ent_ANIM2(ix), #>anim_calabaza_left
    ld  Ent_VX(ix), #-VELOCIDAD_IA_BOUNCER_STROLL
    fin_eje_X:

    ; EJE Y
    ld  a, Ent_Y(ix)
    add Ent_VY(ix)
    cp  #32 + VELOCIDAD_IA_BOUNCER_STROLL
    jp  nc, comprobar_limite_inferior
    ;Límte SUPERIOR
    ld  Ent_VY(ix), #VELOCIDAD_IA_BOUNCER_STOMP
    jp  fin_eje_Y

    comprobar_limite_inferior:
    ld  a, Ent_Y(ix)
    add Ent_VY(ix)
    cp  #200 - HUD_SIZE_H - SPR_SOLDADO_H - VELOCIDAD_IA_BOUNCER_STOMP
    jp  c, fin_eje_Y
    ;Límite INFERIOR
    ld  Ent_VY(ix), #-VELOCIDAD_IA_BOUNCER_STROLL
    fin_eje_Y:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_ia_boss'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (IX) Direcciona de la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; simula un comportamiento de rebote
;; con los limtes del mapa
;;
sys_ia_boss::

    ld  iy, #_m_entities
    ld  a,  Ent_X(iy)
    ld  Ent_OBJX(ix), a
    ld  a,  Ent_Y(iy)
    sub #10
    ld  Ent_OBJY(ix), a
    ld  Ent_AI1(ix), #<sys_ia_move_to_saltar_disparo  
    ld  Ent_AI2(ix), #>sys_ia_move_to_saltar_disparo   
    ld   a, Ent_COMPONENTS(ix)
    sub  #e_cmp_collider
    ld   Ent_COMPONENTS(ix), a
    ret

    sys_ia_boss_en_posicion::

    ld   a, Ent_PATROL2(ix)
    cp   #0
    jp   nz,  boss_en_posicion
        ld   Ent_ANIM1(ix), #<anim_bandido_esconderse
        ld   Ent_ANIM2(ix), #>anim_bandido_esconderse
        ld   Ent_ANIM_COUNT(ix), #IA_ANIM_RUN
        ld   Ent_AI1(ix), #<sys_ia_animacion
        ld   Ent_AI2(ix), #>sys_ia_animacion
        ret
    boss_en_posicion:

    ld   a, (enemies_onscreen)
    cp   #1
    jp   nz, sigo_escondido
        ld   a, (wave_finished)
        cp   #1
        jp   nz, sigo_escondido

        ld   a, Ent_COMPONENTS(ix)
        or   #e_cmp_collider
        ld   Ent_COMPONENTS(ix), a
        ld   Ent_AI1(ix), #<sys_ia_seguir_jugador
        ld   Ent_AI2(ix), #>sys_ia_seguir_jugador
        ld   Ent_AIPREV1(ix), #<sys_ia_seguir_jugador
        ld   Ent_AIPREV2(ix), #>sys_ia_seguir_jugador
        ld   Ent_ANIM_COUNT(ix), #IA_ANIM_RUN
        ld   Ent_PATROL1(ix), #1
        ld   Ent_DIRECCION(ix), #10
        ret
    sigo_escondido:

    call apuntar_jugador
    call sys_ia_shot 

    ret

sys_ia_morir::
    ld  a, Ent_AI_COUNT(ix) ; Recuperamos el contador de espera
    dec a                   ; Y lo decrementamos
    ld  Ent_AI_COUNT(ix), a ; Guardamos su nuevo valor
    cp #0
    jp nz,no_terminar_morir
        call man_game_entity_destroy
        call decrease_enemies
        call sys_collision_enemy_point

        ld   a, (enemies_onscreen)
        or   a
        jp   nz,  no_terminar_morir2
            ld a, (wave_finished)
            or a
            jp z, no_terminar_morir2
            ;; TERMINA
            jp saltar_objeto

        no_terminar_morir2:
        call generar_objeto
        saltar_objeto::

    no_terminar_morir:
    call blink_render_enemy
ret